# -*- coding: utf8 -*-

#-----------------------------------------
# This is a part of Collège 3D project.
# Author:       Pascal Peter
# Copyright:    (C) 2003-2016 Pascal Peter
# Licence:      GNU General Public Licence version 3
# Website:      http://pascal.peter.free.fr/
# Email:        pascal.peter at free.fr
#-----------------------------------------

"""
DESCRIPTION :
    Execution d'un jeu.
"""

# **************************************************
#       IMPORTATION DES MODULES UTILES
# **************************************************
import sys
import os
import time
import math
import mathutils
import glob

import bge

scriptsDir = os.path.abspath(bge.logic.expandPath('//') + '..' + os.sep + 'scripts')
sys.path.insert(0, scriptsDir)
import utils, input_controller, game_controller




# **************************************************
#       CONTRÔLES DU JEU
# **************************************************
controller = None
scene = None
camera = None
player = None
characters = []
old_character = ''
actual_character = ''



# **************************************************
#       FONCTIONS UTILES
# **************************************************
def postDrawTexts():
    """
    affichage des textes.
    """
    utils.postDrawTexts('characters_loop')

def playSound(soundName='clic'):
    if not(bge.logic.globalDict['MENU']['sounds']):
        return
    if soundName == 'jump':
        soundActuator = controller.actuators['Jump']
    else:
        soundActuator = None
    if soundActuator != None:
        controller.activate(soundActuator)

def keyboardManagement(inputState):
    """
    gestion du clavier
    """
    # touches spéciales :
    if inputState['QUIT']:
        doQuitGame()
    elif inputState['TEST']:
        doTest()

def characterManagement(inputState):
    """
    On teste les déplacements à chaque boucle.
    """
    if bge.logic.globalDict['GAME']['state'] != 'characters_loop':
        return
    action = 'wait'
    isMoving = False
    fps = bge.logic.getLogicTicRate()
    if fps < 1:
        fps = 1

    # -------------------------------------------------------------
    # détection des déplacements demandés :
    # -------------------------------------------------------------
    UP = inputState['PLAYER_UP']
    DOWN = inputState['PLAYER_DOWN']
    JUMP = inputState['PLAYER_JUMP']
    turn = input_controller.calcTurn(inputState, player, fps)

    # -----------------------
    # calcul de l'avancée :
    # -----------------------
    maxAdvance = 10
    try:
        (advance, speedAdvance) = player['advance']
    except:
        return
    if UP:
        action = 'walk'
        advance = 1
        if speedAdvance < 5:
            speedAdvance = 5
        else:
            speedAdvance += 2 / fps
            if speedAdvance > maxAdvance:
                speedAdvance = maxAdvance
    elif DOWN:
        action = 'walk'
        advance = - 1
        if speedAdvance < 5:
            speedAdvance = 5
        else:
            speedAdvance += 2 / fps
            if speedAdvance > maxAdvance:
                speedAdvance = maxAdvance
    elif speedAdvance > 0:
        action = 'walk'
        speedAdvance -= 30 / fps
        if speedAdvance < 0:
            speedAdvance = 0
            advance = 0
            action = 'wait'
    player['advance'] = (advance, speedAdvance)
    advance = advance * speedAdvance

    # ------------------
    # calcul du saut :
    # ------------------
    speedJump, maxJump = 1, 20
    (jumping, jump) = player['jump']
    if jumping:
        JUMP = False
    if JUMP:
        jumping = True
        action = 'jump'
        playSound('jump')
        jump = maxJump / fps
    if jumping:
        jump -= speedJump / fps
        if jump <= 0:
            jumping = False
            jump = 0
        else:
            action = 'jump'
    player['jump'] = (jumping, jump)

    # ---------------------------------------------------
    # on ne fait ce qui suit que s'il y a déplacement :
    # ---------------------------------------------------
    player['isMoving'] = False
    if (advance != 0) or (turn != 0) or (jump > 0):
        # calcul de up (pour monter les escaliers) :
        player['isMoving'] = True
        pente, up = 0, 0
        positionEx = player['positionEx']
        position = player.position
        deltaV = position[2] - positionEx[2]
        deltaH = math.hypot((positionEx[0] - position[0]), (positionEx[1] - position[1]))
        if deltaH > 0:
            pente = deltaV / deltaH
        if (pente > 0):
            if pente < 1:
                up = pente * advance
            elif pente < 5:
                up = advance
        player['positionEx'] = (player.position[0], player.position[1], player.position[2])
        # enfin on déplace le personnage :
        advance = advance * 0.01
        up = up * 0.01
        jump = jump * 2
        direction = player.orientation * mathutils.Vector((0, advance, up + jump))
        player['wrapper'].walkDirection = direction
        player.applyRotation([0, 0, turn], True)
        # correction en cas d'erreur 
        # (personnage en dehors des limites, sous le sol, etc) :
        worldLimites = ((-100, 100), (-100, 100))
        if player.position[0] < worldLimites[0][0]:
            player.position[0] = worldLimites[0][0] + 1
        elif player.position[0] > worldLimites[0][1]:
            player.position[0] = worldLimites[0][1] - 1
        if player.position[1] < worldLimites[1][0]:
            player.position[1] = worldLimites[1][0] + 1
        elif player.position[1] > worldLimites[1][1]:
            player.position[1] = worldLimites[1][1] - 1
        if player.position[2] < -5:
            player.position[2] = 20

    # -------------------------------------
    # mise à jour de l'action si besoin :
    # -------------------------------------
    if action != player['action']:
        actionData = utils.actions[action]
        player['action'] = action
        characterName = bge.logic.globalDict['MENU']['character']
        character = scene.objects[characterName]
        theAction = actionData[0]
        character.playAction(theAction, actionData[1], actionData[2], play_mode=actionData[3])

def doQuitGame(nextScene='menu'):
    if bge.logic.globalDict['GAME']['state'] == 'characters_quit':
        return
    bge.logic.globalDict['GAME']['state'] = 'characters_quit'
    bge.logic.globalDict['MENU']['character'] = old_character
    # on libère les fichiers chargés :
    #utils.freeAllLibs()
    utils.freeAllLibs(doAll=True)
    utils.clearGlobalDict()
    # et on retourne au menu (ou à la scène demandée) :
    scene.replace(nextScene)

def doTest():
    global actual_character
    # choix du personnage :
    actual_character = utils.nextInList(
        characters, bge.logic.globalDict['MENU']['character'])
    bge.logic.globalDict['GAME']['state'] = 'characters_first'
    utils.freeAllLibs()#doAll=True)
    scene.restart()



# **************************************************
#       FONCTION PRINCIPALE
# **************************************************
def main():
    # ---------------------------------------------------------
    # RÉCUPÉRATION DE L'ÉTAT DU JEU
    # ---------------------------------------------------------
    # si on lance le fichier directement
    if not('GAME' in bge.logic.globalDict):
        bge.logic.globalDict['GAME'] = {}
        for key in utils.defaultSettingsValues:
            bge.logic.globalDict[key] = utils.defaultSettingsValues[key]
        bge.logic.globalDict['GAME']['pause'] = False
    state = bge.logic.globalDict['GAME'].get('state', '')
    if not(state in ('characters_first', 'characters_loop', 'characters_quit')):
        bge.logic.globalDict['GAME']['state'] = 'characters_first'
    if bge.logic.globalDict['GAME']['state'] == 'characters_quit':
        return

    # ---------------------------------------------------------
    # PREMIER LANCEMENT
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] == 'characters_first':
        # on masque la souris :
        bge.render.showMouse(False)

        global controller, scene, player
        controller = bge.logic.getCurrentController()
        scene = bge.logic.getCurrentScene()
        scene.post_draw += [postDrawTexts]
        bge.logic.globalDict['TEXTS'] = {}
        camera = scene.active_camera
        player = scene.objects['player.characters']
        player['wrapper'] = bge.constraints.getCharacter(player)

        # initialisation des timers :
        bge.logic.globalDict['TEMP']['actualTime'] = 0
        bge.logic.globalDict['TEMP']['actualPos'] = (0, 0)
        bge.logic.globalDict['TEMP']['timeSpecific'] = {4: False}

        # initialisation du personnage :
        global characters, old_character, actual_character
        playerPosition = player.position
        player.position = playerPosition
        if old_character == '':
            old_character = bge.logic.globalDict['MENU']['character']
        characters = []
        root = bge.logic.expandPath('//')
        charsDir = '{1}..{0}chars{0}*.blend'.format(os.sep, root)
        for c in glob.glob(charsDir):
            cc = c.split(os.sep)[-1].split('.')[0]
            if cc != 'kangourou':
                characters.append(cc)
        characters.sort()
        if actual_character == '':
            actual_character = characters[0]
        bge.logic.globalDict['MENU']['character'] = actual_character
        utils.loadChar(actual_character)
        character = scene.objects[actual_character]
        character.position = (playerPosition[0], playerPosition[1], playerPosition[2] - 0.5)
        character.setParent(player)
        player['isMoving'] = False
        player['action'] = ''
        player['positionEx'] = (playerPosition[0], playerPosition[1], playerPosition[2])
        player['advance'] = (0, 0)
        player['jump'] = (False, 0)
        player['outside'] = True

        """
        # le hook (objet porté par le personnage) :
        try:
            player['hook'].endObject()
        except:
            pass
        if mission.hook != None:
            try:
                hook = scene.objects['hook']
            except:
                hook = scene.objects['hook2']
            utils.loadLib(mission.hook[0])
            objet = scene.objectsInactive[mission.hook[1]]
            newObjet = scene.addObject(objet, objet)
            newObjet.position = hook.position
            newObjet.orientation = hook.orientation
            newObjet.setParent(hook)
            player['hook'] = newObjet
        # le chapeau du personnage :
        try:
            player['head'].endObject()
        except:
            pass
        if mission.head != None:
            try:
                head = scene.objects['head']
            except:
                head = scene.objects['head2']
            utils.loadLib(mission.head[0])
            objet = scene.objectsInactive[mission.head[1]]
            newObjet = scene.addObject(objet, objet)
            newObjet.position = head.position
            newObjet.orientation = head.orientation
            newObjet.setParent(head)
            player['head'] = newObjet
        """

        # initialisation terminée :
        player.position = playerPosition
        player['positionEx'] = (playerPosition[0], playerPosition[1], playerPosition[2])
        bge.logic.globalDict['TEMP']['initialTime'] = int(time.time())
        bge.logic.globalDict['GAME']['state'] = 'characters_loop'
        utils.setText(
            'progName', text=actual_character, 
            color=utils.colors['orange'], width=2, position=(0.1, 0.1))

    # ---------------------------------------------------------
    # BOUCLE DES ÉVÉNEMENTS
    # ---------------------------------------------------------
    if bge.logic.globalDict['GAME']['state'] != 'characters_loop':
        return
    inputState = input_controller.inputManagement('game')
    # touche PAUSE :
    if inputState['PAUSE']:
        bge.logic.globalDict['GAME']['pause'] = not(bge.logic.globalDict['GAME']['pause'])
    if bge.logic.globalDict['GAME']['pause']:
        return
    # calculs divers concernant les durées :
    actualTime = int(time.time() - bge.logic.globalDict['TEMP']['initialTime'])

    # gestion des touches spéciales :
    keyboardManagement(inputState)
    if bge.logic.globalDict['GAME']['state'] != 'characters_loop':
        return

    # on teste les déplacements à chaque boucle :
    characterManagement(inputState)
    if bge.logic.globalDict['GAME']['state'] != 'characters_loop':
        return

    # la suite ne sera effectuée que toutes les secondes :
    if actualTime <= bge.logic.globalDict['TEMP']['actualTime']:
        return
    # mise à jour du actualTime :
    bge.logic.globalDict['TEMP']['actualTime'] = actualTime


